#pragma once

#include <limits>
#include <bgtu/computer_graphics_lab_work/renderer_api/color.hpp>
#include <bgtu/computer_graphics_lab_work/renderer_api/point.hpp>
#include <bgtu/computer_graphics_lab_work/renderer_api/sprite.hpp>
#include "sprite_data.hpp"


namespace BGTU::ComputerGraphicsLabWork::Lab1 {
	class PixelGridSprite : public RendererApi::Sprite<Lab1SpriteData> {
	private:
		RendererApi::Color bg, fg;
	public:
		PixelGridSprite(RendererApi::Color bg, RendererApi::Color fg) : bg{bg}, fg{fg} {};

		void draw(BGTU::ComputerGraphicsLabWork::RendererApi::VoxelDrawer *frame, const BGTU::ComputerGraphicsLabWork::Lab1::Lab1SpriteData *data) const override {
			auto centerd = (data->transform * RendererApi::PointF<2>{0, 0});
			auto center = RendererApi::PointI<2>{static_cast<long long>(centerd.x), static_cast<long long>(centerd.y)};

			long long start_x = center.x - ((center.x + data->pixel_size - 1) / data->pixel_size * data->pixel_size);
			long long start_y = center.y - ((center.y + data->pixel_size - 1) / data->pixel_size * data->pixel_size);
			long long w = frame->width();
			long long h = frame->height();
			bool y_flag = ((center.x - start_x) / data->pixel_size + (center.y - start_y) / data->pixel_size) % 2 != 0;
			bool x_flag;
			for (long long y = start_y; y < h; y += data->pixel_size, y_flag = !y_flag) {
				x_flag = y_flag;
				for (long long x = start_x; x < w; x += data->pixel_size, x_flag = !x_flag) {
					auto c = x_flag ? this->bg : this->fg;

					for (long long yy = y + data->pixel_size - 1; yy-- > y;) {
						for (long long xx = x + data->pixel_size - 1; xx-- > x;) {
							frame->add_voxel(xx, yy, std::numeric_limits<double>::max(), c);
						}
					}
				}
			}
		}
	};
}