#pragma once


#include <cstdlib>
#include <QMutex>
#include <QElapsedTimer>
#include <bgtu/computer_graphics_lab_work/renderer_api/sprite.hpp>
#include <bgtu/computer_graphics_lab_work/utilities/matrix.hpp>


namespace BGTU::ComputerGraphicsLabWork::Lab1 {
	class _Lab1SpriteData_Provider;


	struct Lab1SpriteData {
	public:
		Utilities::Matrix3d transform;
		std::size_t pixel_size;
		bool show_grid;

		using Provider = _Lab1SpriteData_Provider;
	};

	class _Lab1SpriteData_Provider : public QObject, public RendererApi::Sprite<Lab1SpriteData>::SpriteDataProvider {
	Q_OBJECT
	private:
		QMutex sync;
		QElapsedTimer time;

		std::size_t pixel_size;
		double radians_per_second;
		RendererApi::VoxelDrawer::visible_pixel_coordinate_fast_t w, h;
		bool show_grid;
	public:
		explicit _Lab1SpriteData_Provider(double radians_per_second) : sync{}, time{}, pixel_size{1}, radians_per_second{radians_per_second}, show_grid{false} {}

		Lab1SpriteData get_sprite_data() override {
			this->sync.lock();
			Lab1SpriteData cached{
					.transform = Utilities::Matrix3d::shift(this->pixel_size / -2.0, this->pixel_size / 2.0) *
								 Utilities::Matrix3d::shift(this->w / 2.0, this->h / 2.0) *
								 Utilities::Matrix3d::rotate(this->time.elapsed() / 1000.0 * this->radians_per_second),
					.pixel_size = this->pixel_size,
					.show_grid = this->show_grid
			};
			this->sync.unlock();
			return cached;
		}

	public slots:
		void set_pixel_size(std::size_t s) {
			this->sync.lock();
			this->pixel_size = s;
			this->sync.unlock();
		}

		void increase_pixel_size() {
			this->sync.lock();
			this->pixel_size++;
			this->sync.unlock();
		}

		void decrease_pixel_size() {
			this->sync.lock();
			if (this->pixel_size > 1)
				this->pixel_size--;
			this->sync.unlock();
		}

		void set_frame_size(RendererApi::VoxelDrawer::visible_pixel_coordinate_fast_t ww, RendererApi::VoxelDrawer::visible_pixel_coordinate_fast_t hh) {
			this->sync.lock();
			this->w = ww;
			this->h = hh;
			this->sync.unlock();
		}

		void set_show_grid(bool f) {
			this->sync.lock();
			this->show_grid = f;
			this->sync.unlock();
		}
	};
}